@@ -1,4 +1,4 @@
-//@line 36 "/mnt/Srcs/Firefox/build/toolkit/components/cookie/content/cookieAcceptDialog.js"
+//@line 37 "/mnt/Firefox/build/toolkit/components/cookie/content/cookieAcceptDialog.js"
 
 const nsICookieAcceptDialog = Components.interfaces.nsICookieAcceptDialog;
 const nsIDialogParamBlock = Components.interfaces.nsIDialogParamBlock;
@@ -123,6 +123,17 @@ function onload()
     } catch (e) {
     }
   }
+
+  // The Private Browsing service might not be available
+  try {
+    var pb = Components.classes["@mozilla.org/privatebrowsing;1"].
+             getService(Components.interfaces.nsIPrivateBrowsingService);
+    if (pb.privateBrowsingEnabled) {
+      var persistCheckbox = document.getElementById("persistDomainAcceptance");
+      persistCheckbox.removeAttribute("checked");
+      persistCheckbox.setAttribute("disabled", "true");
+    }
+  } catch (ex) {}
 }
 
 function showhideinfo()
