@@ -1,4 +1,4 @@
-//@line 42 "/home/karma/src/build/build/toolkit/mozapps/downloads/content/DownloadProgressListener.js"
+//@line 42 "/mnt/Firefox/build/toolkit/mozapps/downloads/content/DownloadProgressListener.js"
 
 /**
  * DownloadProgressListener "class" is used to help update download items shown
@@ -63,9 +63,11 @@ DownloadProgressListener.prototype = {
     } catch (e) { }
   },
 
-  onProgressChange: function(aWebProgress, aRequest, aCurSelfProgress,
-                             aMaxSelfProgress, aCurTotalProgress,
-                             aMaxTotalProgress, aDownload)
+  onProgressChange: function dlPL_onProgressChange(aWebProgress, aRequest,
+                                                   aCurSelfProgress,
+                                                   aMaxSelfProgress,
+                                                   aCurTotalProgress,
+                                                   aMaxTotalProgress, aDownload)
   {
     var download = getDownload(aDownload.id);
 
